@@ -1,11 +1,11 @@
 #############################################################
 #                                                           #
-#	plot.wle.cp function                                #
-#	E-mail: claudio@unive.it                            #
-#	Date: December, 3, 2001                             #
-#	Version: 0.4-1                                      #
+#	plot.wle.cp function                                    #
+#	E-mail: claudio@unive.it                                #
+#	Date: October, 27, 2003                                 #
+#	Version: 0.4-2                                          #
 #                                                           #
-#	Copyright (C) 2001 Claudio Agostinelli              #
+#	Copyright (C) 2003 Claudio Agostinelli                  #
 #                                                           #
 #############################################################
 
@@ -69,50 +69,45 @@ ycoord.best <- ycoord.good[1]
 
 label.best <- label.good[1]
 
-if(length(xcoord.good)==1)
-{
-xcoord.good <- 0
-ycoord.good <- 0
-plot.good <- FALSE
-}else
-{
-xcoord.good <- xcoord.good[-1]
-ycoord.good <- ycoord.good[-1]
-label.good <- label.good[-1]
-plot.good <- TRUE
+if (length(xcoord.good)==1) {
+    xcoord.good <- xcoord.best <- 0
+    ycoord.good <- ycoord.best <- 0
+    plot.good <- FALSE
+} else {
+    xcoord.good <- xcoord.good[-1]
+    ycoord.good <- ycoord.good[-1]
+    label.good <- label.good[-1]
+    plot.good <- TRUE
 }
 
-if(num.max>num.good)
-{
-xcoord.bad <- apply(matrix(wcp.bad[,1:nvar],ncol=nvar),1,sum)[1:min(num.bad,num.max-num.good)]
-ycoord.bad <- wcp.bad[,nvar+1][1:min(num.bad,num.max-num.good)]
-label.bad <- label.bad[1:min(num.bad,num.max-num.good)]
-plot.bad <- TRUE
-}else
-{
-xcoord.bad <- 0
-ycoord.bad <- 0
-plot.bad <- FALSE
+if (num.max>num.good) {
+    xcoord.bad <- apply(matrix(wcp.bad[,1:nvar],ncol=nvar),1,sum)[1:min(num.bad,num.max-num.good)]
+    ycoord.bad <- wcp.bad[,nvar+1][1:min(num.bad,num.max-num.good)]
+    label.bad <- label.bad[1:min(num.bad,num.max-num.good)]
+    plot.bad <- TRUE
+} else {
+    xcoord.bad <- 0
+    ycoord.bad <- 0
+    plot.bad <- FALSE
 }
 
-xlim.min <- min(xcoord.good,xcoord.bad,xcoord.best)
-xlim.max <- max(xcoord.good,xcoord.bad,xcoord.best)
+xlim.min <- min(xcoord.good, xcoord.bad, xcoord.best)
+xlim.max <- max(xcoord.good, xcoord.bad, xcoord.best)
 
 yetichetta <- "WCp"
 
-if(log.scale)
-{
-ycoord.good <- log10(ycoord.good+min(ycoord.good,ycoord.bad,ycoord.best)+1)
-ycoord.bad <- log10(ycoord.bad+min(ycoord.good,ycoord.bad,ycoord.best)+1)
-ycoord.best <- log10(ycoord.best+min(ycoord.good,ycoord.bad,ycoord.best)+1)
-yetichetta <- "WCp log10 scale"
+if (log.scale) {
+    ycoord.good <- log10(ycoord.good+min(ycoord.good,ycoord.bad,ycoord.best)+1)
+    ycoord.bad <- log10(ycoord.bad+min(ycoord.good,ycoord.bad,ycoord.best)+1)
+    ycoord.best <- log10(ycoord.best+min(ycoord.good,ycoord.bad,ycoord.best)+1)
+    yetichetta <- "WCp log10 scale"
 }
 
-ylim.min <- min(ycoord.good,ycoord.bad,ycoord.best)
-ylim.max <- max(ycoord.good,ycoord.bad,ycoord.best)
+ylim.min <- min(ycoord.good, ycoord.bad, ycoord.best)
+ylim.max <- max(ycoord.good, ycoord.bad, ycoord.best)
 
 if (is.null(ylab)) {
-ylab <- yetichetta
+    ylab <- yetichetta
 }
 
 if(plot.it)
